package com.link.core.basic.es.parse;

import com.link.core.basic.es.util.EsAggregateType;
import com.link.core.basic.es.util.EsCacheUtils;
import com.link.core.basic.es.util.EsFieldType;
import com.link.core.basic.es.util.EsUtil;
import com.link.core.basic.model.BasicModel;
import com.link.core.basic.service.BasicServiceException;
import com.link.core.util.StringUtils;
import java.util.Arrays;
import java.util.Map;
import java.util.Objects;
import javax.annotation.Resource;
import org.elasticsearch.script.Script;
import org.elasticsearch.search.aggregations.AggregationBuilder;
import org.elasticsearch.search.aggregations.AggregationBuilders;
import org.elasticsearch.search.aggregations.BucketOrder;
import org.elasticsearch.search.aggregations.bucket.terms.TermsAggregationBuilder;
import org.elasticsearch.search.aggregations.support.ValuesSourceAggregationBuilder;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.springframework.stereotype.Component;

@Component
public class EsAggrParse {
   @Resource
   private EsCacheUtils esCacheUtils;
   public static final String AGGR_SEPARATOR = "-";
   public static final String AGGR_RESULT_NAME = "result";
   public static final String AGGR_RESULT_NESTED_NAME = "nestedResult";
   private static final String[] ES_NUMBER_TYPE;

   public void parseFields(SearchSourceBuilder sourceBuilder, BasicModel basicModel) throws Exception {
      if (StringUtils.isBlank(basicModel.getEsIndex())) {
         throw new BasicServiceException("Es目标索引名称为空");
      } else if (null == basicModel.getGroupFields()) {
         throw new BasicServiceException("聚合查询字段[groupFields]不能为空");
      } else {
         String[] groupFields = basicModel.getGroupFields().split(",");
         if (groupFields.length <= 0) {
            throw new BasicServiceException("聚合查询字段[groupFields]不能为空");
         } else {
            String aggregationType = basicModel.getAggregationType();
            if (StringUtils.isBlank(aggregationType)) {
               throw new BasicServiceException("聚合查询字段[aggregationType]不能为空");
            } else {
               Object groupBuilder;
               String item;
               String aggregationField;
               if (groupFields.length == 1) {
                  Map<String, String> map = EsUtil.parseNestedField(groupFields[0], basicModel);
                  String[] fields = ((String)map.get(EsFieldType.NESTED_OBJECT.getValue())).split("\\.");
                  if (fields.length > 1) {
                     groupBuilder = AggregationBuilders.nested("nestedResult", fields[0]);
                     ((AggregationBuilder)groupBuilder).subAggregation(((TermsAggregationBuilder)AggregationBuilders.terms("result").field(groupFields[0])).size(Integer.MAX_VALUE));
                  } else {
                     groupBuilder = ((TermsAggregationBuilder)AggregationBuilders.terms("result").field(groupFields[0])).size(Integer.MAX_VALUE);
                  }
               } else {
                  StringBuilder esScript = new StringBuilder();
                  aggregationField = "+'-'+";

                  for(int i = 0; i < groupFields.length; ++i) {
                     item = groupFields[i];
                     String fieldType = this.esCacheUtils.getFieldType(basicModel.getEsIndex(), new String[]{item});
                     if (!EsFieldType.KEYWORD.getValue().equals(fieldType)) {
                        item = item + ".keyword";
                     }

                     if (i == groupFields.length - 1) {
                        aggregationField = "";
                     }

                     esScript.append("doc['").append(item).append("']").append(aggregationField);
                  }

                  Script script = new Script(esScript.toString());
                  groupBuilder = ((TermsAggregationBuilder)AggregationBuilders.terms("result").script(script)).size(Integer.MAX_VALUE).order(BucketOrder.key(true));
               }

               aggregationField = basicModel.getAggregationField();
               aggregationField = (String)EsUtil.parseNestedField(aggregationField, basicModel).get(EsFieldType.NESTED_OBJECT.getValue());
               boolean countFlag = EsAggregateType.COUNT.getValue().equals(aggregationType);
               if (StringUtils.isBlank(aggregationField)) {
                  throw new BasicServiceException("求和聚合指定聚合字段[aggregationField]不能为空");
               } else {
                  item = this.esCacheUtils.getFieldType(basicModel.getEsIndex(), new String[]{aggregationField});
                  if (!Arrays.asList(ES_NUMBER_TYPE).contains(item) && !countFlag) {
                     throw new BasicServiceException("该字段类型不支持聚合,字段：" + aggregationField);
                  } else {
                     ValuesSourceAggregationBuilder aggregationBuilder;
                     switch ((EsAggregateType)Objects.requireNonNull(EsAggregateType.getByValue(aggregationType))) {
                        case COUNT:
                           aggregationBuilder = AggregationBuilders.count(EsAggregateType.COUNT.getValue()).field(aggregationField);
                           break;
                        case SUM:
                           aggregationBuilder = AggregationBuilders.sum(EsAggregateType.SUM.getValue()).field(aggregationField);
                           break;
                        case AVG:
                           aggregationBuilder = AggregationBuilders.avg(EsAggregateType.AVG.getValue()).field(aggregationField);
                           break;
                        case MIN:
                           aggregationBuilder = AggregationBuilders.min(EsAggregateType.MIN.getValue()).field(aggregationField);
                           break;
                        case MAX:
                           aggregationBuilder = AggregationBuilders.max(EsAggregateType.MAX.getValue()).field(aggregationField);
                           break;
                        default:
                           throw new BasicServiceException("不支持的聚合操作类型：" + aggregationType);
                     }

                     ((AggregationBuilder)groupBuilder).subAggregation(aggregationBuilder);
                     sourceBuilder.aggregation((AggregationBuilder)groupBuilder);
                     sourceBuilder.size(0);
                  }
               }
            }
         }
      }
   }

   static {
      ES_NUMBER_TYPE = new String[]{EsFieldType.FLOAT.getValue(), EsFieldType.DOUBLE.getValue(), EsFieldType.INTEGER.getValue(), EsFieldType.LONG.getValue()};
   }
}
